// UserTimelineDisplayer.java
//
// Informatics 45 Spring 2010
// Code Example: Parsing and Using Twitter's XML Responses
//
// This program, like our previous code example, is a quick-and-dirty
// experiment, with no exception handling and no organization; it's just
// a main() method that does everything.  But it does illustrate the
// concepts you'd need in order to do this kind of work in your own
// more full-featured programs.

package inf45.spring2010.examples.web3;

import java.io.InputStream;
import java.net.Authenticator;
import java.net.PasswordAuthentication;
import java.net.URL;
import java.net.URLConnection;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


public class UserTimelineDisplayer
{
	public static void main(String[] args)
	throws Exception
	{
		Authenticator.setDefault(
			new Authenticator()
			{
				protected PasswordAuthentication getPasswordAuthentication()
				{
					// To run this program, you'll need to replace the
					// username and password here with one for your own
					// Twitter account.
					return new PasswordAuthentication(
						"myusername", "mypassword".toCharArray());
				}
			});
		
		// The important part of the URL here is the ".xml" part after "user_timeline".
		// This tells Twitter what kind of output we want.  Since we want XML, we say
		// "user_timeline.xml".  If we wanted JSON, another format, we'd say
		// "user_timeline.json" instead.
		URL userTimelineURL = new URL("http://api.twitter.com/1/statuses/user_timeline.xml");
		URLConnection connection = userTimelineURL.openConnection();
		
		// The user_timeline method in the Twitter API is based on an HTTP GET, so
		// we do not need to call setDoOutput() or write to the output stream as
		// we did in the previous code example.  Instead, we fetch the InputStream
		// from our URLConnection, so we can pass it to the XML parser.  This will
		// let us say to the XML parser: "Parse whatever comes across from Twitter."
		InputStream result = connection.getInputStream();

		// This three-statement combination is how you create an XML document using
		// a DOM-based parser.  The reason why you have to write these three lines,
		// specifically, is not extremely important, though you can find plenty of
		// explanation online if you're curious.
		//
		// (Note that the Document class comes from the org.w3c.dom package.  Even
		// though it's name doesn't start with "java", it's still part of the
		// standard Java installation.)
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		Document resultDocument = builder.parse(result);


		// We'll print our output in a couple of ways, just so I can show you a
		// couple of different examples of traversing the XML document once
		// it's loaded.  For an example of what this document will look like,
		// type this URL into your browser:
		//
		//     http://api.twitter.com/1/statuses/user_timeline/myusername.xml
		//
		// (If it doesn't work, replace "myusername" with your own Twitter
		// username and it should work fine.)


		// Here, we'll print just the text of the last few status updates.
		// The only XML element we'll care about is the one called <text>
		// that appears in each <status> element.  We can ask for just
		// the "text" elements by using getElementsByTagName(), which
		// returns us a NodeList.  (Node, in this case, means an XML node,
		// which is a sensible way of describing an XML document if you
		// think of it as a tree.)
		NodeList textNodes = resultDocument.getElementsByTagName("text");

		// This is how you iterate over a NodeList: count from 0 to the
		// length of the list, then ask for each item by calling the item()
		// method.  All we're doing with each node is printing out its
		// text content, which is the free-form text in between the
		// <text> and </text> tags.
		for (int i = 0; i < textNodes.getLength(); i++)
		{
			String text = textNodes.item(i).getTextContent();
			System.out.println(text);
		}


		// This is a slightly more complex way of printing out the information.
		// Instead of just printing out the text, we'll also print out what
		// date and time it was posted.  We'll also "pretty up" the date format
		// a little bit, since the date format specified in the XML is not all
		// that nice to look at: "Fri May 09 01:09:08 +0000 2008".

		// We'll fetch all the <status> nodes, which will let us traverse from
		// one status update to the next.
		NodeList statusNodes = resultDocument.getElementsByTagName("status");

		// When you want to read a string containing a date into a Java object
		// of type Date (which represents a point in time), you need to first
		// specify what format the string is in.  This rather obtuse-looking
		// statement says "You can expect dates to be the first three letters
		// of the name of the day, followed by the first three letters of the
		// name of the month, followed by the day of the month (stretched out to
		// two digits with an extra zero if necessary)..." and so on.  Check
		// out the Java API documentation for SimpleDateFormat if you want the
		// full details of what this string means.
		DateFormat dateFormatter = new SimpleDateFormat("EEE MMM dd HH:mm:ss Z yyyy");
		
		for (int i = 0; i < statusNodes.getLength(); i++)
		{
			Node statusNode = statusNodes.item(i);
			
			Date statusDate = null;
			String text = null;
			
			// Now we iterate through the child elements of each "status"
			// element.  There are only two of them that we care about:
			// <created_at> and <text>; we'll ignore the others.
			NodeList statusChildren = statusNode.getChildNodes();
			
			for (int j = 0; j < statusChildren.getLength(); j++)
			{
				Node childNode = statusChildren.item(j);
				
				if (childNode.getNodeName().equals("text"))
				{
					text = childNode.getTextContent();
				}
				else if (childNode.getNodeName().equals("created_at"))
				{
					statusDate = dateFormatter.parse(childNode.getTextContent());
				}
			}
			
			// Now we'll print out the date and the text, with the date formatted
			// automatically by Java in a way that it determines is appropriate
			// for my locale.  (Since your operating system is generally configured
			// with a time zone, a date format, etc., Java taps into that to make
			// the date look right based on those settings.)
			System.out.println(statusDate.toLocaleString() + ": " + text);
		}
	}
}
